package org.zeroturnaround.javarebel.integration.servlet.weblogic;

import java.io.File;
import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import org.zeroturnaround.javarebel.Logger;
import org.zeroturnaround.javarebel.LoggerFactory;
import org.zeroturnaround.javarebel.RebelServletContext;
import org.zeroturnaround.javarebel.ServletIntegrationFactory;
import org.zeroturnaround.javarebel.integration.servlet.util.IoUtil;
import org.zeroturnaround.javarebel.integration.servlet.util.ServletUtil;

/**
 * Copies JSP and Tag files from JavaRebel working directory to Weblogic working directory.
 * <p>
 * Thanks to Sylvain Laurent.
 * 
 * @author Rein Raudjärv
 */
public class JspUpdateUtil {

  private static final Logger log = LoggerFactory.getInstance();

  public static void handleJspUpdate(ServletContext ctx, File rootTempDir, ServletRequest req, ServletResponse resp) {
    try {
      String path = ServletUtil.getUriFromRequest((HttpServletRequest) req);
      handleJspUpdateInternal(ctx, rootTempDir, path);
    } catch (Throwable e) {
      log.log("Webloigc JSP update could not be handled");
      log.error(e);
    }
  }
  
  public static void handleJspUpdate(ServletContext ctx, File rootTempDir, String path) {
    try {
      handleJspUpdateInternal(ctx, rootTempDir, path);
    } catch (Throwable e) {
      log.log("Webloigc JSP update could not be handled");
      log.error(e);
    }
  }
  
  /**
   * Copies JSP or Tag file from JavaRebel working directory to Weblogic working directory.
   * 
   * @param ctx used to invoke method {@link ServletContext#getRealPath(String)}
   *               to get the JSP location provided by JavaRebel.
   * @param rootTempDir Weblogic temporary directory.
   * @param path URI of the JSP requested 
   */
  private static void handleJspUpdateInternal(ServletContext ctx, File rootTempDir, String path) {
    if (!ServletIntegrationFactory.getInstance().isResourceReplaced((RebelServletContext) ctx, path)){
      // No rebel.xml virtual mapping, so no need to copy the files
      return;
    }
    
    // Skip if no JSP page or Tag file
    String lc = path.toLowerCase();
    if (!lc.endsWith(".jsp") && !lc.endsWith(".jspx")
        && !lc.endsWith(".tag") && !lc.endsWith(".tagx"))
      return;
    
    // Weblogic's temporary JSP file
    File warTempDir = new File(rootTempDir, "war");
    File tempFile = new File(warTempDir, path);
    File parentDir = tempFile.getParentFile();
    if (!parentDir.exists() && !parentDir.mkdirs())
      return; // Could not create the parent directory, giving up
    
    // JavaRebel's JSP file path
    String realPath = ctx.getRealPath(path);
    if (realPath == null)
      return; // JavaRebel is not handling this resource
    
    // JavaRebel's JSP file with the timestamp
    File realFile = new File(realPath);
    long realDate = realFile.lastModified();
    
    if (realDate == 0) {
      // Removed
      if (tempFile.exists()) {
        if (tempFile.delete())
          if (log.isEnabled())
            log.log("JSP file '" + path + "' removed.");
          else if (log.isEnabled())
            log.log("Could not remove JSP file '" + path + "'.");
      }
    }
    else {
      // Maybe updated
      if (realDate > tempFile.lastModified()) {
        IoUtil.copy(realFile, tempFile);
        if (log.isEnabled())
          log.log("JSP file '" + path + "' updated.");
      }
    }
  }
  
}
